<?php
/**
 * OBERON_HotelReservation - Widget class.
 * 
 * This file is part of the OBERON System and WordPress plugins made by Exalogic company and is released under the same license.
 * For more information please see oberon_hotel_reservation_plugin.php.
 * 
 * Copyright (c) 2007-2020 Mário Moravcik. All rights reserved.
 * 
 * Version: 1.4.4
 */

class OBERON_HotelReservation_Admin {

	/**
	 * Setting name shared for all plugin classes.
	 */
	const CONST_SERVICE_URL_NAME = 'service-url';
	const CONST_CALCULATE_ACCOMMODATION_NAME = 'calc-accommodation';
	const CONST_CALCULATE_LOCALTAX_NAME = 'calc-localtax';
	const CONST_CALCULATE_SERVICES_NAME = 'calc-services';
	const CONST_ROOM_TYPE_CHANGE_NAME = 'roomtype-change';
	const CONST_PRICE_CATEGORY_GUEST_NAME = 'price-category-guest';

	/**
	 * Required WordPress version.
	 *
	 * @var str $required_version .
	 */
	private $required_version = '4.7';

	/**
	 * Local settings.
	 *
	 * @var array
	 */
	private $plugin_settings = array();

	/**
	 * Create widget instance.
	 */
	public function __construct() {
		if ( ! defined( 'ABSPATH' ) ) {
			return;
		}

		add_action( 'admin_menu', array( $this, 'add_menu' ) );
		add_action( 'plugins_loaded', array( $this, 'load_textdomain' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_filter( 'plugin_action_links', array( $this, 'add_settings_link' ), 10, 2 );
		
		//add_action( 'before_wp_oberon-reservation', array( $this, 'show_version_warning' ) );

		$plugin_settings = array();		
		$this->version = 0;

		$this->load_settings();

	}

	/** ----------------------------------------------------------------------------------------------- */
	/** ----------------------------------------------------------------------------------------------- */
	/** ------  P R O P E R T I E S    -------------------------- */
	/** ----------------------------------------------------------------------------------------------- */
	/** ----------------------------------------------------------------------------------------------- */

	/**
	 * Property value get API url.
	 */
	public function get_oberon_web_url() {
		return $this->settings->oberon_web_url->value;
	}

	/**
	 * Property value get accommodation calculation flag.
	 */
	public function get_reservation_calc_accommodation() {
		return $this->settings->calculate_accommodation->value;
	}

	/**
	 * Property value get local tax calculation flag.
	 */
	public function get_reservation_calc_localtax() {
		return $this->settings->calculate_localtax->value;
	}

	/**
	 * Property value get basic services calculation flag.
	 */
	public function get_reservation_calc_services() {
		return $this->settings->calculate_services->value;
	}

	/**
	 * Property value allow room type change flag.
	 */
	public function get_allow_roomtype_change() {
		return $this->settings->allow_roomtype_change->value;
	}

	/**
	 * Property value price category guest for calculation.
	 */
	public function get_price_category_guest() {
		return $this->settings->price_category_guest->value;
	}

	/**
	 * Get the OBERON Web widget version property.
	 *
	 * @param string $format .
	 */
	public function widget_version( $format = true ) {
		$ver = OBERONRESERVATION_VERSION;
		if ( $format ) {
			$ver = substr( number_format( $ver, 0, '', '.' ), 0, 3 );
		}
		return $ver;
	}

	/**
	 * Load widget text domain.
	 */
	public function load_textdomain() {
		//$locale = apply_filters( 'plugin_locale', get_locale(), OBERONRESERVATION_TEXT_DOMAIN );
		load_plugin_textdomain( OBERONRESERVATION_TEXT_DOMAIN, false, basename( dirname( __FILE__ ) ) . '/languages' );
	}

	/** ----------------------------------------------------------------------------------------------- */
	/** ----------------------------------------------------------------------------------------------- */
	/** ------  A D M I N   S E T T I N G S    -------------------------- */
	/** ----------------------------------------------------------------------------------------------- */
	/** ----------------------------------------------------------------------------------------------- */

	/**
	 * Register admin scripts.
	 *
	 * @param Obj $page .
	 */
	function enqueue_scripts( $page ) {

		wp_enqueue_style( 'exalogic-oberon-admin-styles', OBERONRESERVATION_URL . 'css/exalogic-oberon-styles-admin.css', array(), '1.0' );

		wp_enqueue_script( 'oberon-application-base', OBERONRESERVATION_URL . 'js/exalogic-oberon-application-base.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker' ), '1.0', true );
		wp_enqueue_script( 'jquery-lingua', OBERONRESERVATION_URL . 'js/jquery.lingua.min.js', array( 'jquery' ), '1.0', true );
		wp_enqueue_script( 'exalogic-oberon-reservation-plugin-admin', OBERONRESERVATION_URL . 'js/exalogic-oberon-reservation-plugin-admin.js', array( 'jquery' ), '1.0' );

	}

	/**
	 * Add page in admin menu.
	 */
	public function add_menu() {
		// page_title, menu_title, capability, menu_slug, function.
		add_options_page(
			'OBERON - Hotel reservation',
			'OBERON - Hotel reservation',
			'manage_options',
			'exalogic-oberon-hotel-reservation',
			array( $this, 'settings_page_handler' )
		);

	}

	/**
	 * Add a link to the settings page.
	 *
	 * @param array  $links .
	 * @param string $file .
	 */
	public function add_settings_link( $links, $file ) {
		if ( 'exalogic_oberon_hotel_reservations/exalogic_oberon_hotel_reservation_pulugin.php' && current_user_can( 'manage_options' === $file ) ) {
			$settings_link = sprintf( '<a href="%s">%s</a>', admin_url( 'options-general.php?page=exalogic-oberon-hotel-reservation' ), __( 'Settings', OBERONRESERVATION_TEXT_DOMAIN ) );
			array_unshift( $links, $settings_link );
		}

		return $links;
	}

	/**
	 * Get link to edit settings.
	 *
	 * @param string $text .
	 * @param array  $links .
	 */
	public function get_settings_link( $text, $links = null ) {
		$settings_link = sprintf( '<a href="%s">%s</a>', admin_url( 'options-general.php?page=exalogic-oberon-hotel-reservation' ), $text );
		if ( $links && ! empty( $links ) ) {
			array_unshift( $links, $settings_link );
		}
		return $settings_link;
	}

	/**
	 * Include settings page.
	 */
	public function settings_page_handler() {
		if ( ! defined( 'OBERONRESERVATION_ADMIN_PAGE' ) ) {
			define( 'OBERONRESERVATION_ADMIN_PAGE', true );
		}

		include_once( OBERONRESERVATION_PATH . 'inc/settings_page.php' );
	}

	/**
	 * Get default plugin settings.
	 */
	function get_default_plugin_settings() {
		return array(
			'service-url' => '',
			'calc-accommodation' => 1,
			'calc-localtax' => 0,
			'calc-services' => 0,
			'roomtype-change' => 1,
			'price-category-guest' => '',
		);
	}

	/**
	 * Load widget settings.
	 */
	public function load_settings() {
		if ( empty( $plugin_settings ) ) {
			// load if empty.		
			$plugin_settings = get_option( OBERONRESERVATION_OPTION_NAME, $this->get_default_plugin_settings() );
		}

		// load defaults if the options don't exist...
		if ( empty( $plugin_settings ) ) {
			$plugin_settings = $this->get_default_plugin_settings();
		}

		$this->version             = get_option( OBERONRESERVATION_OPTION_VERSION, 0 ); //stored version for later updates.

		$this->settings = (object) array(
			'oberon_web_url' => (object) array(
				'control_id' => 'oberon-web-url',
				'value_type' => 'text',
				'value' => '',
			),
			'price_category_guest' => (object) array(
				'control_id' => 'price-category-guest',
				'value_type' => 'text',
				'value' => '',
			),
			'allow_roomtype_change' => (object) array(
				'control_id' => 'allow-roomtype-change',
				'value_type' => 'numeric[0/1]',
				'value' => 1,
			),
			'calculate_accommodation' => (object) array(
				'control_id' => 'calculate-accommodation',
				'value_type' => 'numeric[0/1]',
				'value' => 1,
			),
			'calculate_localtax' => (object) array(
				'control_id' => 'calculate-localtax',
				'value_type' => 'numeric[0/1]',
				'value' => 0,
			),
			'calculate_services' => (object) array(
				'control_id' => 'calculate-services',
				'value_type' => 'numeric[0/1]',
				'value' => 0,
			),
		);

		$this->settings->oberon_web_url->value              = untrailingslashit( $plugin_settings[ OBERON_HotelReservation_Admin::CONST_SERVICE_URL_NAME ] );
		if ( array_key_exists( OBERON_HotelReservation_Admin::CONST_CALCULATE_ACCOMMODATION_NAME, $plugin_settings ) ) {
			$this->settings->calculate_accommodation->value     = $plugin_settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_ACCOMMODATION_NAME ];
		}
		if ( array_key_exists( OBERON_HotelReservation_Admin::CONST_CALCULATE_LOCALTAX_NAME, $plugin_settings ) ) {
			$this->settings->calculate_localtax->value          = $plugin_settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_LOCALTAX_NAME ];
		}
		if ( array_key_exists( OBERON_HotelReservation_Admin::CONST_CALCULATE_SERVICES_NAME, $plugin_settings ) ) {
			$this->settings->calculate_services->value          = $plugin_settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_SERVICES_NAME ];
		}
		if ( array_key_exists( OBERON_HotelReservation_Admin::CONST_ROOM_TYPE_CHANGE_NAME, $plugin_settings ) ) {
			$this->settings->allow_roomtype_change->value       = $plugin_settings[ OBERON_HotelReservation_Admin::CONST_ROOM_TYPE_CHANGE_NAME ];
		}
		if ( array_key_exists( OBERON_HotelReservation_Admin::CONST_PRICE_CATEGORY_GUEST_NAME, $plugin_settings ) ) {
			$this->settings->price_category_guest->value        = untrailingslashit( $plugin_settings[ OBERON_HotelReservation_Admin::CONST_PRICE_CATEGORY_GUEST_NAME ] );
		}

	}

	/**
	 * Save widget setting.
	 */
	public function save_settings() {
		check_admin_referer( 'oberon-settings-save-order' );
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die('Access denied');
		}

		// $user = wp_get_current_user();
		// $allowed_roles = array( 'editor', 'administrator', 'author' );
		// if ( array_intersect( $allowed_roles, $user->roles ) ) {
		// 	return false;
		// }

		$this->load_settings();
		$defaults = $this->get_default_plugin_settings();
		$settings = array();

		if ( isset( $_POST['oberon-settings-save'] ) ) {

			$url = esc_url_raw( $_POST[$this->settings->oberon_web_url->control_id] );
			$calc_acc = (sanitize_text_field( $_POST[$this->settings->calculate_accommodation->control_id] ) !== '1') ? 0 : 1;
			$calc_lct = (sanitize_text_field( $_POST[$this->settings->calculate_localtax->control_id] ) !== '1') ? 0 : 1;
			$calc_srv = (sanitize_text_field( $_POST[$this->settings->calculate_services->control_id] ) !== '1') ? 0 : 1;
			$calc_rtc = (sanitize_text_field( $_POST[$this->settings->allow_roomtype_change->control_id] ) !== '1') ? 0 : 1;
			$pricecat = sanitize_text_field( $_POST[$this->settings->price_category_guest->control_id] );

			$settings = $plugin_settings; // get actual settings

			$settings[ OBERON_HotelReservation_Admin::CONST_SERVICE_URL_NAME ] = $url;
			$settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_ACCOMMODATION_NAME ] = $calc_acc;
			$settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_LOCALTAX_NAME ] = $calc_lct;
			$settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_SERVICES_NAME ] = $calc_srv;
			$settings[ OBERON_HotelReservation_Admin::CONST_ROOM_TYPE_CHANGE_NAME ] = $calc_rtc;
			$settings[ OBERON_HotelReservation_Admin::CONST_PRICE_CATEGORY_GUEST_NAME ] = $pricecat;

		} else {
			return false;
		}

		// Save the new settings.
		update_option( OBERONRESERVATION_OPTION_NAME, $settings );

		$plugin_settings = array();
		$this->load_settings();
		return true;
	}

}
